from __future__ import annotations

from typing import Any

from ares_datamodel.ares_struct_pb2 import QuantityValue
from ares_datamodel.quantity_validation import validate_quantity_value
from pint import Quantity, UnitRegistry
from pint.facets.plain import PlainUnit

_UNIT_REGISTRY = UnitRegistry()


def get_unit_registry() -> UnitRegistry:
    return _UNIT_REGISTRY


def to_pint_quantity(quantity_value: QuantityValue, registry: UnitRegistry | None = None) -> Quantity:
    """
    Convert an ARES QuantityValue to a Pint quantity.
    """
    ureg = registry or _UNIT_REGISTRY
    quantity = quantity_value.scalar * ureg.parse_units(quantity_value.unit)
    validate_quantity_value(quantity_value, registry=ureg)
    return quantity


def from_pint_quantity(
    quantity: Quantity,
    unit: str | None = None,
    registry: UnitRegistry | None = None,
) -> QuantityValue:
    ureg = registry or _UNIT_REGISTRY
    converted = quantity.to(ureg.parse_units(unit)) if unit else quantity
    resolved_unit = _format_units_name(converted.units)
    return QuantityValue(
        scalar=_magnitude_to_float(converted.magnitude),
        unit=resolved_unit,
    )


def convert_quantity_value_unit(
    quantity_value: QuantityValue,
    target_unit: str,
    registry: UnitRegistry | None = None,
) -> QuantityValue:
    ureg = registry or _UNIT_REGISTRY
    source = to_pint_quantity(quantity_value, registry=ureg)
    return from_pint_quantity(source, unit=target_unit, registry=ureg)


def _format_units_name(unit: PlainUnit):
    unit_str = f"{unit:~}"
    unit_str = unit_str.replace("**", "^")
    unit_str = "".join(unit_str.split())
    return unit_str


def _magnitude_to_float(magnitude: Any) -> float:
    if isinstance(magnitude, complex):
        if magnitude.imag != 0:
            raise ValueError("Complex magnitude is not supported for QuantityValue scalar conversion.")
        return float(magnitude.real)
    return float(magnitude)



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'scripting/symbol_kind.proto')
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bscripting/symbol_kind.proto\x12\x18ares.datamodel.scripting*\xef\x01\n\nSymbolKind\x12\x1b\n\x17SYMBOL_KIND_UNSPECIFIED\x10\x00\x12\x16\n\x12SYMBOL_KIND_DEVICE\x10\x01\x12\x18\n\x14SYMBOL_KIND_FUNCTION\x10\x02\x12\x18\n\x14SYMBOL_KIND_VARIABLE\x10\x03\x12\x16\n\x12SYMBOL_KIND_STRUCT\x10\x04\x12\x17\n\x13SYMBOL_KIND_KEYWORD\x10\x05\x12\x17\n\x13SYMBOL_KIND_PLANNER\x10\x06\x12\x18\n\x14SYMBOL_KIND_ANALYZER\x10\x07\x12\x14\n\x10SYMBOL_KIND_TYPE\x10\x08b\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scripting.symbol_kind_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_SYMBOLKIND']._serialized_start = 58
    _globals['_SYMBOLKIND']._serialized_end = 297


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'scripting/completion.proto')
_sym_db = _symbol_database.Default()
from ..scripting import script_symbol_metadata_pb2 as scripting_dot_script__symbol__metadata__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1ascripting/completion.proto\x12\x18ares.datamodel.scripting\x1a&scripting/script_symbol_metadata.proto"\xd1\x01\n\x0eCompletionItem\x12\r\n\x05label\x18\x01 \x01(\t\x12\x13\n\x0binsert_text\x18\x02 \x01(\t\x12F\n\x12insert_text_format\x18\x03 \x01(\x0e2*.ares.datamodel.scripting.InsertTextFormat\x12\x11\n\tsort_text\x18\x04 \x01(\t\x12@\n\x08metadata\x18\x05 \x01(\x0b2..ares.datamodel.scripting.ScriptSymbolMetadata*U\n\x10InsertTextFormat\x12!\n\x1dINSERT_TEXT_FORMAT_PLAIN_TEXT\x10\x00\x12\x1e\n\x1aINSERT_TEXT_FORMAT_SNIPPET\x10\x01b\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scripting.completion_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_INSERTTEXTFORMAT']._serialized_start = 308
    _globals['_INSERTTEXTFORMAT']._serialized_end = 393
    _globals['_COMPLETIONITEM']._serialized_start = 97
    _globals['_COMPLETIONITEM']._serialized_end = 306